@@ -10,7 +10,7 @@
 
 import re
 
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 __all__ = [
     'BlockGrammar', 'BlockLexer',
@@ -155,6 +155,24 @@ class BlockGrammar(object):
 class BlockLexer(object):
     """Block level lexer for block grammars."""
 
+    default_features = [
+        'newline', 'block_code', 'fences', 'heading',
+        'nptable', 'lheading', 'hrule', 'block_quote',
+        'list_block', 'block_html', 'def_links',
+        'def_footnotes', 'table', 'paragraph', 'text'
+    ]
+
+    list_features = (
+        'newline', 'block_code', 'fences', 'lheading', 'hrule',
+        'block_quote', 'list_block', 'block_html', 'text',
+    )
+
+    footnote_features = (
+        'newline', 'block_code', 'fences', 'heading',
+        'nptable', 'lheading', 'hrule', 'block_quote',
+        'list_block', 'block_html', 'table', 'paragraph', 'text'
+    )
+
     def __init__(self, rules=None, **kwargs):
         self.options = kwargs
 
@@ -174,12 +192,7 @@ class BlockLexer(object):
         src = src.rstrip('\n')
 
         if not features:
-            features = (
-                'newline', 'block_code', 'fences', 'heading',
-                'nptable', 'lheading', 'hrule', 'block_quote',
-                'list_block', 'block_html', 'def_links',
-                'def_footnotes', 'table', 'paragraph', 'text'
-            )
+            features = self.default_features
 
         def manipulate(src):
             for key in features:
@@ -251,11 +264,6 @@ class BlockLexer(object):
         self.tokens.append({'type': 'list_end'})
 
     def _process_list_item(self, cap, bull):
-        features = (
-            'newline', 'block_code', 'fences', 'lheading', 'hrule',
-            'block_quote', 'list_block', 'block_html', 'text',
-        )
-
         cap = self.rules.list_item.findall(cap)
 
         _next = False
@@ -290,7 +298,7 @@ class BlockLexer(object):
 
             self.tokens.append({'type': t})
             # recurse
-            self.parse(item, features)
+            self.parse(item, self.list_features)
             self.tokens.append({'type': 'list_item_end'})
 
     def parse_block_quote(self, m):
@@ -335,13 +343,7 @@ class BlockLexer(object):
                 newlines.append(line[whitespace:])
             text = '\n'.join(newlines)
 
-        features = (
-            'newline', 'block_code', 'fences', 'heading',
-            'nptable', 'lheading', 'hrule', 'block_quote',
-            'list_block', 'block_html', 'table', 'paragraph', 'text'
-        )
-
-        self.parse(text, features)
+        self.parse(text, self.footnote_features)
 
         self.tokens.append({
             'type': 'footnote_end',
@@ -461,6 +463,13 @@ class InlineGrammar(object):
 class InlineLexer(object):
     """Inline level lexer for inline grammars."""
 
+    default_features = [
+        'escape', 'autolink', 'url', 'tag',
+        'footnote', 'link', 'reflink', 'nolink',
+        'double_emphasis', 'emphasis', 'code',
+        'linebreak', 'strikethrough', 'text',
+    ]
+
     def __init__(self, renderer, rules=None, **kwargs):
         self.options = kwargs
 
@@ -471,6 +480,12 @@ class InlineLexer(object):
 
         if not rules:
             rules = InlineGrammar()
+            if self.options.get('hard_wrap'):
+                # {2,} -> *
+                rules.linebreak = re.compile(r'^ *\n(?!\s*$)')
+                rules.text = re.compile(
+                    r'^[\s\S]+?(?=[\\<!\[_*`~]|https?://| *\n|$)'
+                )
 
         self.rules = rules
 
@@ -487,14 +502,11 @@ class InlineLexer(object):
 
     def output(self, src, features=None):
         src = src.rstrip('\n')
+        if self.options.get('escape'):
+            src = escape(src)
 
         if not features:
-            features = [
-                'escape', 'autolink', 'url', 'tag',
-                'footnote', 'link', 'reflink', 'nolink',
-                'double_emphasis', 'emphasis', 'code',
-                'linebreak', 'strikethrough', 'text',
-            ]
+            features = list(self.default_features)
 
         if self._in_footnote and 'footnote' in features:
             features.remove('footnote')
@@ -621,7 +633,8 @@ class InlineLexer(object):
         return self.renderer.strikethrough(text)
 
     def output_text(self, m):
-        return escape(m.group(0))
+        text = m.group(0)
+        return self.renderer.text(text)
 
 
 class Renderer(object):
@@ -765,6 +778,13 @@ class Renderer(object):
         """
         return '<del>%s</del>' % text
 
+    def text(self, text):
+        """Rendering unformatted text.
+
+        :param text: text content.
+        """
+        return escape(text)
+
     def autolink(self, link, is_email=False):
         """Rendering a given link or email address.
 
@@ -848,7 +868,8 @@ class Renderer(object):
 
         :param text: contents of all footnotes.
         """
-        return '<div class="footnotes">\n<ol>%s</ol>\n</div>\n' % text
+        html = '<div class="footnotes">\n%s<ol>%s</ol>\n</div>\n'
+        return html % (self.hrule(), text)
 
 
 class Markdown(object):
